-- edu_test.lua

require "celutil";
require "textlayout";
require "format";

require "CXBox";

celestia:log("edu_test");

showOverlay = true;

----------------------------------------------------------------------
--
-- LOCAL FUNCTIONS 
--
----------------------------------------------------------------------

local beginScreenDraw =
	function()
		gl.MatrixMode(gl.PROJECTION);
		gl.PushMatrix();
		gl.LoadIdentity();
		sx,sy = celestia:getscreendimension();
	    gl.Ortho(0, sx, 0, sy, 0, 1); 
    	gl.MatrixMode(gl.MODELVIEW);
		gl.PushMatrix();
		gl.LoadIdentity();
		gl.Disable(gl.TEXTURE_2D);
		gl.Disable(gl.LIGHTING);
				gl.Enable(gl.TEXTURE_2D);
				gl.Enable(gl.BLEND);
				gl.BlendFunc(gl.SRC_ALPHA, gl.ONE_MINUS_SRC_ALPHA);
	end;

local endScreenDraw =
	function()
	    gl.MatrixMode(gl.PROJECTION);
	    gl.PopMatrix();
	    gl.MatrixMode(gl.MODELVIEW);
	    gl.PopMatrix();
	end;


zoomToFit = 
	function()
		local obs = celestia:getobserver()
		local obsPos = obs:getposition()
		local sel = celestia:getselection()
		if sel == nil or sel:name() == "?" then return end
		local selPos = sel:getposition()
		local dist = obsPos:distanceto(selPos)
		local rad = sel:radius()
		local appDia = 2*math.atan(rad/dist)
		obs:setfov(appDia*1.20)
		celestia:flash("Zoom to fit "..sel:name(), 2) 
	end;
	
gotoToFit = 
	function()
		local obs = celestia:getobserver()
		local sel = celestia:getselection()
		local fov = obs:getfov()
		if empty(sel) == false then
			obs:follow(sel);
			obs:gotodistance(sel, 5.5*sel:radius()/(fov/0.4688))
		end
	end;

----------------------------------------------------------------------

MLY_PER_KM = 1/9466411.842;

empty =
	function(obj)
		return 	obj == nil or obj:name() == "?";
	end

changeOrbitDistance =
	function(d)
		center = obs:getframe():getrefobject();
	    if empty(center) or not empty(sel) then
	        center = sel;
	        obs:setframe(celestia:newframe(obs:getframe():getcoordinatesystem(), center));
	    end
		if not empty(center) then
			obsPos = obs:getposition();
			focusPosition = center:getposition();
			size = center:radius();
	
			minOrbitDistance = size*MLY_PER_KM;
			naturalOrbitDistance = (4.0 * minOrbitDistance);
	
			v = focusPosition:vectorto(obsPos);
			currentDistance = v:length();
	
			if currentDistance < minOrbitDistance then
				minOrbitDistance = currentDistance * 0.5;
			end
	
			if currentDistance >= minOrbitDistance and naturalOrbitDistance ~= 0 then
				r = (currentDistance - minOrbitDistance) / naturalOrbitDistance;
				newDistance = minOrbitDistance + naturalOrbitDistance * math.exp(math.log(r) + d);
				v = v * (newDistance / currentDistance);
				newPos = focusPosition:addvector( v );
				obs:setposition(newPos);
			end
		end    
	end

----------------------------------------------------------------------

dolly = {};

dolly.Speed = .2;
dolly.Rate  =  0;

dolly.start =
	function(dr)
		if dr == dolly.Rate then
			dolly.Rate = 0;
			return;
		end;
		
		obs = celestia:getobserver();
		sel = celestia:getselection();
		if empty(sel) then 
			dolly.Rate = 0;
		else
			dolly.Rate = dr;
		end
	end

dolly.tick =
	function(dt)
		changeOrbitDistance( dt*dolly.Rate );
	end;

dolly.in2 =
	function()
		dolly.start(-dolly.Speed);
	end

dolly.out =
	function()
		dolly.start(dolly.Speed);
	end   			

----------------------------------------------------------------------

overlay = {};

overlay.visible = true;

overlay.toggle = 
	function()
		overlay.visible = not overlay.visible;
	end;
	
overlay.draw =
	function()
		if not overlay.visible then return end;
		screenw, screenh = celestia:getscreendimension();

		beginScreenDraw();
			screenBox:updatebounds();
			screenBox:drawall();
		endScreenDraw();

	end;
	
----------------------------------------------------------------------
--
-- HOOK EVENT HANDLER
--
----------------------------------------------------------------------

keymap = 
	{
 		-- i = dolly.in2;
		-- o = dolly.out;
		-- z = zoomToFit;
		-- q = os.exit;
		I = overlay.toggle;
		G = gotoToFit;
	 }

edu_test = 
{

	mousebuttondown =
		function(self,x,y,b)
			if eventeater then
				if eventeater:mousebuttondown(x,sy-y,b) then
					return true;
				end;
			end;
			sx,sy = celestia:getscreendimension();
			c = screenBox:findcontainer(x,sy-y);
			if c then
				return c:mousebuttondown(x,sy-y,b);
			end;
		   return false;
		end;
	
	keydown =
		function(self,ch,cmods)
			ch = string.char(ch);
			if keymap[ch] then 
				return true;
			else
				return false;
			end;
		end;
				
	resize =
		function(self,x,y)
			screenBox:updatebounds();
			screenBox:adjustborders();
		   return false;
		end;
		
	mousebuttonmove =
		function(self,dx,dy,b)
			sx,sy = celestia:getscreendimension();
			if eventeater then
				return eventeater:mousebuttonmove(dx,-dy,b);
			end;
		   return false;
		end;
	
	mousebuttonup =
		function(self,x,y,b)
			sx,sy = celestia:getscreendimension();
			if eventeater then
				return eventeater:mousebuttonup(x,sy-y,b);
			end;
		   return false;
		end;
		
	charentered =
		function(self,ch)
			if eventeater then
				result = eventeater:charentered(ch);
				if result then return true end
			end;
		  local f = keymap[ch]
		   if f then 
			 f() 
			  return true;
		   end
		   return false;
		end;
	
	renderoverlay =
		function(self)
			sx,sy = celestia:getscreendimension();
			screenw,screenh = sx,sy;
	
			if not titlefont then
				titlefont = celestia:gettitlefont();
				normalfont = celestia:getfont();
				textlayout:init();
				textlayout:setfont(normalfont);
				textlayout:setlinespacing(20);
				require "toolBox";
			end;
			
			overlay.draw();
		
			if lastTime == 0 then lastTime = celestia:getscripttime() end;
			currentTime = celestia:getscripttime();
			if dolly.Rate ~= 0 then
				dolly.tick( currentTime-lastTime );
			end	
			lastTime = currentTime;
	
		end;
	
};

----------------------------------------------------------------------

--[[
	tf = celestia:getparamstring("TitleFont");
	titlefont = celestia:loadfont("fonts/"..tf);

-- titlefont = celestia:loadfont("fonts/helvbold12.txf"); 

	textlayout:init();
	textlayout:setfont(titlefont);
	textlayout:setlinespacing(20);
--]]

require "screenBox";
screenBox.Toplevel = true;

-- require "toolBox";

celestia:log("edu_test");

print("edu_test");

