require "locale";

width4, height4 = celestia:getscreendimension();
yScreenRatio4 = 722/height4;

-- Define the compass image to be used.
compImageFilename = "extras/edu_test/locale/"..lang.."/images/compass_"..lang..".png";
compassImage = celestia:loadtexture(compImageFilename);
if not(compassImage) then
	compImageFilename = "extras/edu_test/images/compass.png";
	compassImage = celestia:loadtexture(compImageFilename);
end

-- Define some initial values.
bgcolor = false;
displonglat = true;
switchable = false;

local cbutextoff = {0.7, 0.7, 1, 0.6};
local cbubordoff = {0.2, 0.2 , 0.6, 0.4};

source = celestia:getselection();

-- Rotate the compass image.
-- A translation is also needed to make the image rotate about its center. 
rot = function (x,y,w)
	x = x-width4/2;
	y = y-42-80/2-20;
	local x2 = x*math.cos(w) - y*math.sin(w);
	local y2 = y*math.cos(w) + x*math.sin(w);
	x2 = x2+width4/2;
	y2 = y2+42+80/2+20;
	return x2, y2;
end

-- Define the function that draws the compass image.
drawcompass = function ()
	local az_rad = az*3.14159/180;
	lb,bb,rb,tb = compassFrameBox:bounds();
	local xlb, ylb = rot(lb, bb, az_rad);
	local xrb, yrb = rot(rb, bb, az_rad);
	local xrt, yrt = rot(rb, tb, az_rad);
	local xlt, ylt = rot(lb, tb, az_rad);
	compassImage:bind();
	gl.Enable(gl.TEXTURE_2D);
	gl.Begin(gl.QUADS);
	gl.Color(1, 1, 1, 0.5);
	gl.TexCoord(0, 1); gl.Vertex(xlb, ylb);
	gl.TexCoord(1, 1); gl.Vertex(xrb, yrb);
	gl.Color(1, 1, 1, 0.5);
	gl.TexCoord(1, 0); gl.Vertex(xrt, yrt);
	gl.TexCoord(0, 0); gl.Vertex(xlt, ylt);
	gl.End();
	gl.Disable(gl.TEXTURE_2D);
end

compassBox = CXBox:new()
	:init(0, 0, 0, 0)
	:text(_("Compass"))
	--:bordercolor({1, 1, 0, 1})
	--:fillcolor({0.2, 0, 0, 0.3})
	:movable(false)
	:active(false)
	:attach(screenBox, (width4-128)/2, 5, (width4-128)/2, height4-128-5)

compassBox.Customdraw = 
	function(this)
		-- Define the limit distance for which the current selection becomes the new source object.
		local sel = celestia:getselection();
		local obs = celestia:getobserver();
		local distfromsel = celutil.get_dist(obs, sel);
		local sel_radius = sel:radius();
		local distfromselsurf = distfromsel - sel_radius;
		local distfromsource = celutil.get_dist(obs, source);
		local source_radius = source:radius();
		local distfromsourcesurf = distfromsource - source_radius
		if empty(source) == true or (source ~= sel and math.abs(distfromselsurf) < math.abs(distfromsourcesurf)) then
			source = sel;
		end
		--celestia:print(source:name().." "..distfromselsurf.."   "..distfromsourcesurf);
		if source:type() == "location" then
			source = source:getinfo().parent;
		end
		az, elev = celutil.get_az_elev (obs, source);
		longlatBox.Visible = true;
		if distfromsource < 1.02*source_radius then
			-- Planetarium mode.
			planetariumBox.Visible = false;
			longlatBox.Active = true;
			compassFrameBox.Text = "^";
			compassFrameBox.Active = true;
			if switchable == false then
				displonglat = false;
				switchable = true;
			end
			-- Avoid bug when selecting the Milky Way.
			if source:type() ~= "deepsky" then
				celutil.force_up(obs, source);
			end
			-- Avoid bug when centering source in planetarium mode.
			if elev < -89 then
				local vec = celestia:newvector(1,0,0);
				local rot = celestia:newrotation(vec, math.pi/2);
				obs:rotate(rot); -- Set elevation back to 0.
			end
			-- Call the function that draws the compass.
			drawcompass();
		else
			-- Navigation mode.
			compassFrameBox.Text = "";
			compassFrameBox.Active = false;
			compassBox.Fillcolor = nil;
			bgcolor = false;
			if empty(source) == true then
				planetariumBox.Visible = false;
				longlatBox.Visible = false;
				textlayout:printinbox(_("No current selection."), compassBox, 0, 0);
			else
				planetariumBox.Visible = true;
				longlatBox.Active = false;
				displonglat = true;
				switchable = false;
			end
		end
		-- Make the size of the compass independant from the screen size.
		width4, height4 = celestia:getscreendimension();
		yScreenRatio4 = 722/height4;
		compassBox:attach(screenBox, (width4-128)/2, 20, (width4-128)/2, height4-128-20)
	end;

compassFrameBox = CXBox:new()
	:init(0, 0, 0, 0)
	--:bordercolor(cbubordoff)
	:textfont(titlefont)
	:textcolor({1,1,1,0.3})
	:textpos("center")
	:movetext(0, -17)
	:text("^")
	:movable(false)
	:active(true)
	:attach(compassBox, 24, 42, 24, 6)
	
compassFrameBox.Action = (function()
	return
		function()
			-- Switch to a red background color for a better readability of the compass box.
			bgcolor = not(bgcolor);
			if bgcolor == true then
				compassBox.Fillcolor = {0.2, 0, 0, 0.3};
			else
				compassBox.Fillcolor = nil;
			end
		end
	end) ();

longlatBox = CXBox:new()
	:init(0, 0, 0, 0)
	--:bordercolor(cbubordoff)
	:movable(false)
	:active(true)
	:attach(compassBox, 5, 8, 5, 87)
	
longlatBox.Customdraw = 
	function(this)
		-- Switch between longitude-latitude and azimuth-elevation string display.
		textlayout:setfont(normalfont);
		textlayout:setlinespacing(normalfont:getheight());
		textlayout:setfontcolor({0.8, 0.8, 0.9, 0.8});
		if displonglat == true then
			local obs = celestia:getobserver();
			local long, lat = celutil.get_long_lat(obs, source);
			long_str = _("Longitude")..": "..string.format("%4.2f°", long);
			lat_str = _("Latitude")..": "..string.format("%3.2f°", lat);
			textlayout:setpos(this.lb+12, this.tb-13);
			textlayout:println(long_str);
			textlayout:println(lat_str);
		else
			az_str = _("Azimuth")..": "..string.format("%4.2f°", az);
			elev_str = _("Elevation")..": "..string.format("%3.2f°", elev);
			textlayout:setpos(this.lb+17, this.tb-13);
			textlayout:println(az_str);
			textlayout:println(elev_str);
		end
	end;

longlatBox.Action = (function()
	return
		function()
			displonglat = not(displonglat)
		end
	end) ();

planetariumBox = CXBox:new()
	:init(0, 0, 0, 0)
	:textfont(normalfont)
	:movetext(0, 5)
	:text(_("planetarium mode"))
	--:textcolor({0.8, 0.8 , 0.9, 0.5})
	--:bordercolor({0.6, 0.6 , 0.6, 0.4})
	:textcolor(cbutextoff)
	:bordercolor(cbubordoff)
	:movable(false)
	:active(true)
	:attach(compassBox, 0, 45, 0, 65)
	:fittotext(0)
	
planetariumBox.Action = (function()
	return
		function()
			-- Switch to planetarium mode.
			local obs = celestia:getobserver();
			obs:goto(source);
			obs:gotosurface(source);
		end
	end) ();