require "locale";

width4, height4 = celestia:getscreendimension();
yScreenRatio4 = 722/height4;

-- Define the compass string.
-- For localized versions that needs 2 bytes characters,
-- please use the standard compassBox version.
local ds = " "..string.rep(". ", 7);
local S = _("S");
local W = _("W");
local N = _("N");
local E = _("E");
--local compass_str = "  . . . . . . . S/W . . . . . . .  W  . . . . . . . N/W . . . . . . .  N  . . . . . . . N/E . . . . . . .  E  . . . . . . . S/E . . . . . . .  S";
local compass_str = " "..ds..S.."/"..W..ds.." "..W.." "..ds..N.."/"..W..ds.." "..N.." "..ds..N.."/"..E..ds.." "..E.." "..ds..S.."/"..E..ds.." "..S;
local strlen = string.len(compass_str);
local Npos = strlen/2;

-- Define some initial values
bgcolor = false;
displonglat = true;
switchable = false;

local cbutextoff = {0.7, 0.7, 1, 0.6};
local cbubordoff = {0.2, 0.2 , 0.6, 0.4};

source = celestia:getselection();

compassBox2 = CXBox:new()
	:init(0, 0, 0, 0)
	:text(_("Compass"))
	--:bordercolor({1, 1, 0, 1})
	--:fillcolor({0.2, 0, 0, 0.3})
	:movable(false)
	:active(true)
	:attach(screenBox, (width4-290)/2, 0.04, (width4-290)/2, -0.07*yScreenRatio4)

compassBox2.Customdraw = 
	function(this)
		-- Define the limit distance for which the current selection becomes the new source object. 
		local sel = celestia:getselection();
		local obs = celestia:getobserver();
		local distfromsel = celutil.get_dist(obs, sel);
		local sel_radius = sel:radius();
		local distfromselsurf = distfromsel - sel_radius;
		local distfromsource = celutil.get_dist(obs, source);
		local source_radius = source:radius();
		local distfromsourcesurf = distfromsource - source_radius
		if empty(source) == true or (source ~= sel and math.abs(distfromselsurf) < math.abs(distfromsourcesurf)) then
			source = sel;
		end
		--celestia:print(source:name().." "..distfromselsurf.."   "..distfromsourcesurf);
		if source:type() == "location" then
			source = source:getinfo().parent;
		end
		az, elev = celutil.get_az_elev (obs, source);
		if empty(source) == true then
			gotoSurfaceBox.Visible = false;
			longlatBox.Visible = false;
			textlayout:printinbox(_("No current selection."), compassBox2, 0, 0);
		else
			longlatBox.Visible = true;
			if distfromsource < 1.05*source_radius then
				-- Planetarium mode.
				gotoSurfaceBox.Visible = false;
				longlatBox.Active = true;
				if switchable == false then
					displonglat = false;
					switchable = true;
				end
				-- Avoid bug when selecting the Milky Way.
				if source:type() ~= "deepsky" then
					celutil.force_up(obs, source);
				end
				-- Avoid bug when centering source in planetarium mode.
				local look = celutil.lookat(obs):normalize();
				if math.abs(look.x) >= 0.99 then
					local vec = celestia:newvector(1,0,0);
					local rot = celestia:newrotation(vec, math.pi/2);
					obs:rotate(rot); -- Set elevation back to 0.
				end
				-- Display a sub compass string centered in the direction where the observer is looking.
				angle = az;
				if angle > 180 then
					angle = angle-360;
				end
				local dirpos = math_round(Npos + angle*strlen/360);
				comp_str = string.sub(compass_str, dirpos-19, dirpos+19);
				if dirpos < 20 then
					comp_str = string.sub(compass_str, 144-19+dirpos)..string.sub(compass_str, 1, dirpos+19);
				end
				if dirpos > 124 then
					comp_str = string.sub(compass_str, dirpos-19, 144)..string.sub(compass_str, 1, 19-144+dirpos);
				end
				textlayout:setfont(titlefont);
				textlayout:setfontcolor({0.8, 0.8, 0.9, 0.6});
				textlayout:printinbox(comp_str, compassBox2, 0, -15);
			else
				-- Navigation mode.
				gotoSurfaceBox.Visible = true;
				longlatBox.Active = false;
				displonglat = true;
				switchable = false;
			end
		end
		-- Make the size of the compass independant from the screen size.
		width4, height4 = celestia:getscreendimension();
		yScreenRatio4 = 722/height4;
		compassBox2:attach(screenBox, (width4-290)/2, 0.04, (width4-290)/2, -0.07*yScreenRatio4)
	end;

compassBox2.Action = (function()
	return
		function()
			-- Switch to a red background color for a better readability of the compass box.
			bgcolor = not(bgcolor);
			if bgcolor == true then
				compassBox2.Fillcolor = {0.2, 0, 0, 0.3};
			else
				compassBox2.Fillcolor = nil;
			end
		end
	end) ();
	
longlatBox = CXBox:new()
	:init(0, 0, 0, 0)
	--:bordercolor(cbubordoff)
	:movable(false)
	:active(true)
	:attach(compassBox2, 35, 5, 35, 30)
	
longlatBox.Customdraw = 
	function(this)
		-- Switch between longitude-latitude and azimuth-elevation string display.
		textlayout:setfont(normalfont);
		textlayout:setfontcolor({0.8, 0.8, 0.9, 0.8});
		textlayout:setpos(this.lb+5, this.tb-11);
		if displonglat == true then
			local obs = celestia:getobserver();
			local long, lat = celutil.get_long_lat(obs, source);
			long_str = _("Longitude").." : "..string.format("%4.2f°", long);
			lat_str = _("Latitude").." : "..string.format("%3.2f°", lat);
			textlayout:println(long_str);
			textlayout:setpos(this.lb+128, this.tb-11);
			textlayout:println(lat_str);
		else
			az_str = _("Azimuth").." : "..string.format("%4.2f°", az);
			elev_str = _("Elevation").." : "..string.format("%3.2f°", elev);
			textlayout:println(az_str);
			textlayout:setpos(this.lb+125, this.tb-11);
			textlayout:println(elev_str);
		end
	end;

longlatBox.Action = (function()
	return
		function()
			displonglat = not(displonglat)
		end
	end) ();

gotoSurfaceBox = CXBox:new()
	:init(0, 0, 0, 0)
	:textfont(normalfont)
	:movetext(0, 6)
	:text(_("planetarium mode"))
	--:textcolor({0.8, 0.8 , 0.9, 0.5})
	--:bordercolor({0.6, 0.6 , 0.6, 0.4})
	:textcolor(cbutextoff)
	:bordercolor(cbubordoff)
	:movable(false)
	:active(true)
	:attach(compassBox2, 0, 0.6, 0, -0.3)
	:fittotext(0)
	
gotoSurfaceBox.Action = (function()
	return
		function()
			-- Switch to planetarium mode.
			local obs = celestia:getobserver();
			obs:goto(source);
			obs:gotosurface(source);
		end
	end) ();