require "locale";

local months = { _("Jan"), _("Feb"), _("Mar"), _("Apr"), _("May"), _("Jun"), _("Jul"), _("Aug"), _("Sep"), _("Oct"), _("Nov"), _("Dec")};

width1, height1 = celestia:getscreendimension();
yScreenRatio = 722/height1;

local cbutextoff = {0.7, 0.7, 1, 0.6};
local cbutexton = {0.8, 0, 0, 0.5};
local cbubordoff = {0.2, 0.2 , 0.6, 0.4};
local cbubordon = {0.6, 0, 0, 0.3};

function setPosition(timescale)
	if timescale >= 0 then
		pos = (math.pow(timescale, 1/timeratePower))/1.2;
	else
		pos = -(math.pow(math.abs(timescale), 1/timeratePower))/1.2;
	end
end

timescale = celestia:gettimescale();
timeratePower = 3;
setPosition(timescale);

firstStart = true;

timezone = 0;
zonename = "UTC";

timeBox = CXBox:new()
	:init(0, 0, 0, 0)
	:text(_("Time"))
	--:bordercolor({0, 0, 0, 0})
	--:fillcolor({.2,.2,.6,.2})
	:movable(false)
	:attach(screenBox, width1-165-2, 0.79, 2, -0.14*yScreenRatio)

timeBox.Customdraw = 
	function(this)
		local time = celestia:gettime();
		time = time + timezone/24;
		local timescale = celestia:gettimescale();
		local date = celestia:fromjulianday(time);
		-- We have to set some limit values for 'time' to avoid bugs with Julian days...
		if time < 1 then
			celestia:settime(1);
			celestia:settimescale(1);
		elseif time > 2e9 then
			celestia:settime(2e9);
			celestia:settimescale(1);
		end
		textlayout:setfont(normalfont);
		textlayout:setpos(this.lb, this.tb-15);
		textlayout:setlinespacing(normalfont:getheight());
		textlayout:setfontcolor({0.8, 0.8, 0.9, 0.9});
		if date_format == "day/month/year" then
			textlayout:println(_("Date :").." "..date.day..' '..months[date.month]..' '..date.year);
		elseif date_format == "year/month/day" then
			textlayout:println(_("Date :").." "..date.year..' '..months[date.month]..' '..date.day);
		else
			textlayout:println(_("Date :").." "..months[date.month]..' '..date.day..' '..date.year);
		end
		textlayout:println(_("Time :").." "..date.hour..':'..format:twodigits(date.minute)..':'..format:twodigits(math.floor(date.seconds)));
		--textlayout:println(_("JD :").. " "..time);
		if math.abs(timescale) < 1 then
			if math.abs(timescale) <= 1e-15 then
				timescale_str = _("time stopped");
			else
				timescale_str = string.format("(%1.0e)", timescale).."x";
			end
		else
			timescale_str = string.format("%0.0f", timescale).."x";
		end
		textlayout:setfontcolor(cbutextoff);
		textlayout:setpos(this.lb+normalfont:getwidth(_("Rate :"))+5, this.tb-44);
		textlayout:println(" "..timescale_str);
		width1, height1 = celestia:getscreendimension();
			yScreenRatio = 722/height1;
			timeBox:attach(screenBox, width1-165-2, 0.79, 2, -0.14*yScreenRatio);
	end;		 

slideFrame = CXBox:new()
	:bordercolor({0.8, 0.8, 1, 0.4})
	--:fillcolor({.2,.2,.6,.1})
	:attach(timeBox, 25, 26, 10, 61)
	:visible(true)
	
slideFrame.Customdraw = 
	function(this)
		lb,bb,rb,tb = slideFrame:bounds();
		lb2,bb2,rb2,tb2 = slideBox:bounds();
		slowMax = math.pow(((rb-lb-16)/2)*1.2, 1);
		mediumMax = math.pow(((rb-lb-16)/2)*1.2, 3);
		fastMax = math.pow(((rb-lb-16)/2)*1.2, 5);
		value = lb2-60-lb;
		value = math.pow(value*1.2, timeratePower);
		if lastvalue ~= value and firstStart == false then
			lastvalue = value;
			celestia:settimescale(value);
		end
		firstStart = false;
	end;

slideBox = CXBox:new()
	:bordercolor({0.2, 0.2 , 0.6, 0.6})
	:fillcolor({0.2, 0.2, 0.6, 0.3})
	:attach(slideFrame, 60+pos, 0, 60-pos, 0)
	:visible(true)
	:movable(true)
	
slideBox.Customdraw = 
	function(this)
	local timescale = celestia:gettimescale();
	setPosition(timescale);
	if lastpos ~= pos then
		lastpos = pos;
		if math.abs(timescale) > slowMax and math.abs(timescale) <= mediumMax and timeratePower == 1 then
			mediumButton.Action();
		end
		if math.abs(timescale) > mediumMax and math.abs(timescale) <= fastMax and timeratePower == 3 then
			fastButton.Action();
		end
		if math.abs(timescale) > fastMax and timeratePower == 5 then
			if timescale > 0 then
				celestia:settimescale(fastMax);
			else
				celestia:settimescale(-fastMax);
			end
		end
		if math.abs(timescale) <= fastMax then
			setPosition(timescale);
			slideBox:attach(slideFrame, 60+pos, 0, 60-pos, 0);
		end
	end
end;

timerateBox = CXBox:new()
	--:bordercolor({0.8, 0.8, 1, 0.4})
	--:fillcolor({.2,.2,.6,.1})
	:attach(timeBox, 0, 54, 50, 33)
	:textfont(normalfont)
	:textcolor(cbutextoff)
	--:textpos("center")
	:movetext(-5, 6)
	:text(_("Rate :"))
	:visible(true)
	:active(true)

timerateBox.Action = (function()
	return
		function()
			slideFrame.Visible = not(slideFrame.Visible);
			slowButton.Visible = not(slowButton.Visible);
			mediumButton.Visible = not(mediumButton.Visible);
			fastButton.Visible = not(fastButton.Visible);
			resetButton.Visible = not(resetButton.Visible);
		end
	end) ();
	
	
slowButton = CXBox:new()
	:bordercolor(cbubordoff)
	:attach(timeBox, 30, 5, 100, 83)
	:textfont(normalfont)
	:textcolor(cbutextoff)
	:textpos("center")
	:movetext(0, 7)
	:text(_("slow"))
	:visible(true)
	:active(true)
				
slowButton.Action = (function()
	return
		function()
			timeratePower = 1;
			slowButton.Bordercolor = cbubordon;
			mediumButton.Bordercolor = cbubordoff;
			fastButton.Bordercolor = cbubordoff;
			slowButton.Textcolor = cbutexton;
			mediumButton.Textcolor = cbutextoff;
			fastButton.Textcolor = cbutextoff;
		end
	end) ();
	
mediumButton = CXBox:new()
	:bordercolor(cbubordon)
	:attach(timeBox, 72, 5, 58, 83)
	:textfont(normalfont)
	:textcolor(cbutexton)
	:textpos("center")
	:movetext(0, 7)
	:text(_("med."))
	:visible(true)
	:active(true)
				
mediumButton.Action = (function()
	return
		function()
			timeratePower = 3;
			slowButton.Bordercolor = cbubordoff;
			mediumButton.Bordercolor = cbubordon;
			fastButton.Bordercolor = cbubordoff;
			slowButton.Textcolor = cbutextoff;
			mediumButton.Textcolor = cbutexton;
			fastButton.Textcolor = cbutextoff;
		end
	end) ();
				
fastButton = CXBox:new()
	:bordercolor(cbubordoff)
	:attach(timeBox, 115, 5, 15, 83)
	:textfont(normalfont)
	:textcolor(cbutextoff)
	:textpos("center")
	:movetext(0, 7)
	:text(_("fast"))
	:visible(true)
	:active(true)
				
fastButton.Action = (function()
	return
		function()
			timeratePower = 5;
			slowButton.Bordercolor = cbubordoff;
			mediumButton.Bordercolor = cbubordoff;
			fastButton.Bordercolor = cbubordon;			
			slowButton.Textcolor = cbutextoff;
			mediumButton.Textcolor = cbutextoff;
			fastButton.Textcolor = cbutexton;
		end
	end) ();
	
resetButton = CXBox:new()
	:bordercolor(cbubordoff)
	:attach(timeBox, 3, 26, 146, 61)
	:textfont(normalfont)
	:textcolor(cbutextoff)
	:textpos("center")
	:movetext(0, 6)
	:text("1x")
	:visible(true)
	:active(true)

resetButton.Action = (function()
	return
		function()
			--slideBox:attach(slideFrame, 60, 0, 60, 0);
			celestia:settimescale(1);
		end
	end) ();

timeZoneButton = CXBox:new()
	--:bordercolor(cbubordoff)
	:textfont(normalfont)
	:textcolor(cbutextoff)
	--:textpos("center")
	:movetext(-2, 6)
	:text(zonename)
	:attach(timeBox, 90, 68, 22, 18)
	:visible(true)
	:active(true)

timeZoneButton.Customdraw = 
	function(this)
		if timezone ~= 0 then
			zonename = "GMT"..timezone_str;
		else
			zonename = "UTC";
		end
		timeZoneButton.Text = zonename;
	end;
				
timeZoneButton.Action = (function()
	return
		function()
			timeZoneBox.Visible = not(timeZoneBox.Visible);
			if timeZoneBox.Visible == true then
				timeZoneButton.Textcolor = cbutexton;
			else
				timeZoneButton.Textcolor = cbutextoff;
			end
		end
	end) ();

timeZoneBox = CXBox:new()
	--:bordercolor({0.2, 0.2 , 0.6, 0.6})
	:fillcolor({0.2, 0.2 , 0.6, 0.1})
	:attach(screenBox, width1-300, 0.875, 170, -0.05*yScreenRatio)
	:visible(false)
	:active(false)
				
timeZoneBox.Customdraw = 
	function(this)
		if timeBox.Visible == false then
			timeZoneButton.Action();
		end
		width1, height1 = celestia:getscreendimension();
		yScreenRatio = 722/height1;
		timeZoneBox:attach(screenBox, width1-300, 0.875, 170, -0.05*yScreenRatio);
		textlayout:setfontcolor({0.9, 0.9, 1, 0.6});
		timezone_str = string.format("%+i", timezone);
		textlayout:setpos(this.lb+5, this.tb-13);
		textlayout:println(_("Time Zone").." : ");
		textlayout:setpos(this.lb+65, this.tb-30);
		textlayout:println("GMT"..timezone_str);
	end;

timeZonePreviousButton = CXBox:new()
	:bordercolor(cbubordoff)
	:textfont(normalfont)
	:textcolor(cbutextoff)
	:textpos("center")
	:movetext(0, 7)
	:text("<")
	:attach(timeZoneBox, 21, 3, 92, 20)
	:visible(true)
	:active(true)
				
timeZonePreviousButton.Action = (function()
	return
		function()
			timezone = timezone-1;
			if timezone < -12 then
				timezone = 12;
			end
		end
	end) ();

timeZoneNextButton = CXBox:new()
	:bordercolor(cbubordoff)
	:textfont(normalfont)
	:textcolor(cbutextoff)
	:textpos("center")
	:movetext(0, 7)
	:text(">")
	:attach(timeZoneBox, 40, 3, 73, 20)
	:visible(true)
	:active(true)
				
timeZoneNextButton.Action = (function()
	return
		function()
			timezone = timezone+1;
			if timezone > 12 then
				timezone = -12;
			end
		end
	end) ();
