-- toolBox.lua

require "CXBox";

local width0, height0 = celestia:getscreendimension();

local cbutextoff = {1,1,1, 0.3};
local cbutexton = {0.8, 0, 0, 0.5};
local cbubordoff = {0.8, 0.8, 0.8, 0.2};
local cbubordon = {0.6, 0, 0, 0.3};

makeToolBox =
	function(tools)
		charheight = 13;
		margin = 4;
		maxwidth = 5;
		for i,v in tools do
			textwidth = normalfont:getwidth(v.Text);
			if textwidth > maxwidth then maxwidth = textwidth end;
			numitems = i;
		end;
		
		buttonWidth = maxwidth+2*margin;
		buttonHeight = (charheight+2*margin);
		columnWidth = buttonWidth+margin;
		boxWidth = numitems * (buttonWidth+margin) + margin;
		boxHeight = buttonHeight+2*(margin+2);
		newToolBox = CXBox:new()
			:init(0,0,0,0)
			--:fillcolor({.1,.1,.1,.1})
			--:bordercolor({0.2, 0.2, 0.2, 0.9})
			:attach(screenBox, (width0-boxWidth)/2, height0-boxHeight-3, (width0-boxWidth)/2, 7)
			:movable(false)

		newToolBox.Customdraw = 
			function(this)
				width0, height0 = celestia:getscreendimension();
				--if width0 ~= lastwidth0 or height0 ~= lastheight0 then
					--lastwidth0 = width0;
					--lastheight0 = height0;
					newToolBox:attach(screenBox, (width0-boxWidth)/2, height0-boxHeight-3, (width0-boxWidth)/2, 7);
				--end
			end;

		column = margin;
		for i,v in tools do
			local vv = v;
			local testButton = CXBox:new()
				:text(v.Text)
				:textpos("center")
				:movetext(0, 5)
				:textfont(normalfont)
				:textcolor(cbutextoff)
				:bordercolor(cbubordoff)
				:attach(newToolBox, column, 0.20, -buttonWidth, -0.6)
				:movable(false)
				:visible(true)
				:active(true)
				:editable(false)
				
			testButton.Action = (function() return function()
									vv.Visible = not vv.Visible;
									if vv.Visible == true then
										testButton.Bordercolor = cbubordon;
										testButton.Textcolor = cbutexton;
									else
										testButton.Bordercolor = cbubordoff;
										testButton.Textcolor = cbutextoff;
									end
										
									end
								end) ();
	
			column = column+columnWidth;
		end;
	return newToolBox;
	end;
	

require "toolset";

toollist = {};
for k,v in ipairs(toolset) do
	if pcall( function() require (v) end ) then
		require(v);
		table.insert(toollist, _G[v]);
		_G[v].Visible = false;
	end
end;

toolBox = makeToolBox(toollist);

